/*
  Synchronisierte Stoppuhren zur Bestimmung der Schallgeschwindigkeit

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


p5.disableFriendlyErrors = true; // disables FES

// LAYOUT
let mn;
let button_width;
let button_height;
let button_y;
let text_size
let line_width;
let button_round;

let button_l_pressed;
let button_r_pressed;

const button_l_c1 = [75, 252, 128];
const button_r_c1 = [75, 252, 217];
const button_l_c2 = [244, 12, 43];
const button_r_c2 = [244, 97, 12];

let mouse_x_min_l;
let mouse_x_max_l;
let mouse_y_min_l;
let mouse_y_max_l;
let mouse_x_min_r;
let mouse_x_max_r;
let mouse_y_min_r;
let mouse_y_max_r;

let layout_line1_start_x;
let layout_line1_end_x;
let layout_line1_y;
let layout_line2_start_x;
let layout_line2_end_x;
let layout_line2_y;
let layout_line3_start_x;
let layout_line3_end_x;
let layout_line3_y;
let layout_line4_start_x;
let layout_line4_end_x;
let layout_line4_y;


// GLOBAL
let stopwatch_l_running;
let stopwatch_r_running;

let start_time;
let end_time_l;
let end_time_r;

let inp;
let inp_dist;




function setup() {
  createCanvas(windowWidth, windowHeight);

  stopwatch_l_running = false;
  stopwatch_r_running = false;
  start_time = 0;
  end_time_l = 0;
  end_time_r = 0;

  inp = createInput('100');
  inp.input(myInputEvent);
  inp_dist = 100;

  calculateLayout();

  stopwatch_l_running = false;
  stopwatch_r_running = false;
  start_time = 0;
  end_time_l = 0;
  end_time_r = 0;
}


function draw() {
  background(255);

  if (stopwatch_l_running)
    end_time_l = (millis() - start_time) * 0.001;

  if (stopwatch_r_running)
    end_time_r = (millis() - start_time) * 0.001;

  renderLayout();
  renderButtons();
}


function calculateLayout() {
  mn = width * height * 0.000014;
  text_size = height * 0.07;
  line_width = mn * 0.4;
  button_width = (width - 4 * mn) * 0.5;
  button_height = height - 6 * mn - 3 * text_size;
  button_y = 2 * mn + text_size;
  button_round = 4 * mn;

  mouse_x_min_l = mn;
  mouse_x_max_l = mn + button_width;
  mouse_y_min_l = button_y;
  mouse_y_max_l = button_y + button_height;
  mouse_x_min_r = button_width + mn * 3;
  mouse_x_max_r = width - mn;
  mouse_y_min_r = button_y;
  mouse_y_max_r = button_y + button_height;

  layout_line1_start_x = mn;
  layout_line1_end_x = button_width - mn * 2;
  layout_line1_y = mn * 4 + text_size * 2 + button_height;
  layout_line2_start_x = width * 0.5 - mn * 2;
  layout_line2_end_x = width * 0.5 + mn * 2;
  layout_line2_y = mn * 3 + text_size * 2 + button_height;
  layout_line3_start_x = layout_line2_start_x;
  layout_line3_end_x = layout_line2_end_x;
  layout_line3_y = mn * 4 + text_size * 2 + button_height;
  layout_line4_start_x = width - text_size - mn;
  layout_line4_end_x = width - mn;
  layout_line4_y = layout_line1_y;

  button_l_pressed = false;
  button_r_pressed = false;

  stroke(0);
  strokeWeight(line_width);
  strokeCap(ROUND);

  fill(255);

  textSize(text_size);
  textAlign(CENTER);
  textFont('Avenir')
  textStyle(NORMAL);

  inp.position(mn * 2, mn * 3 + text_size + button_height);
  inp.size(button_width - mn * 7 - text_size * 1.5, text_size * 0.7);
  inp.style('text-align: center');
  inp.style('font-size: ' + str(text_size * 0.9) + 'px');
  inp.style('font-weight: bold');
}


function renderLayout() {
  push();
  line(layout_line1_start_x, layout_line1_y, layout_line1_end_x, layout_line1_y);

  line(layout_line2_start_x, layout_line2_y, layout_line2_end_x, layout_line2_y);
  line(layout_line3_start_x, layout_line3_y, layout_line3_end_x, layout_line3_y);

  line(layout_line4_start_x, layout_line4_y, layout_line4_end_x, layout_line4_y);

  fill(0);
  strokeWeight(line_width * 0.5);
  text(end_time_l.toFixed(2), mn + button_width * 0.5, mn + text_size);
  text(end_time_r.toFixed(2), (width + button_width) * 0.5 + mn, mn + text_size);

  let delta_time = abs(end_time_l - end_time_r);
  if (!stopwatch_l_running || !stopwatch_r_running)
    text((inp_dist / delta_time).toFixed(2), (width + button_width - text_size) * 0.5 + mn * 2, mn * 4 + text_size * 2.25 + button_height);
  else
    text("Infinity", (width + button_width - text_size) * 0.5 + mn * 2, mn * 4 + text_size * 2.25 + button_height);
  textSize(text_size * 0.9);
  text(delta_time.toFixed(2), button_width * 0.5 - mn * 3, mn * 5 + text_size * 2.75 + button_height);

  textSize(text_size);
  text("s", button_width - mn * 5, mn * 5 + text_size * 2.75 + button_height);
  text("m", button_width - mn * 5, mn * 3 + text_size * 1.75 + button_height);

  text("m", width - text_size * 0.5 - mn, mn * 3 + text_size * 2 + button_height);
  text("s", width - text_size * 0.5 - mn, mn * 5 + text_size * 2.5 + button_height);

  textAlign(RIGHT);
  text("s", button_width, mn + text_size);
  text("s", width - mn * 2, mn + text_size);

  pop();
}


function checkButtons() {
  if (touches.length > 1) {
    if (touches[0].x < touches[1].x) {
      if (touches[0].x > mouse_x_min_l && touches[0].x < mouse_x_max_l &&
        touches[0].y > mouse_y_min_l && touches[0].y < mouse_y_max_l &&
        !button_l_pressed) {
        button_l_pressed = true;
        switchStopWatchL();
      }
      if (touches[1].x > mouse_x_min_r && touches[1].x < mouse_x_max_r &&
        touches[1].y > mouse_y_min_r && touches[1].y < mouse_y_max_r &&
        !button_r_pressed) {
        button_r_pressed = true;
        switchStopWatchR();
      }

    } else {
      if (touches[1].x > mouse_x_min_l && touches[1].x < mouse_x_max_l &&
        touches[1].y > mouse_y_min_l && touches[1].y < mouse_y_max_l &&
        !button_l_pressed) {
        button_l_pressed = true;
        switchStopWatchL();
      }
      if (touches[0].x > mouse_x_min_r && touches[0].x < mouse_x_max_r &&
        touches[0].y > mouse_y_min_r && touches[0].y < mouse_y_max_r &&
        !button_r_pressed) {
        button_r_pressed = true;
        switchStopWatchR();
      }

    }
  } else {
    if (mouseX > mouse_x_min_l && mouseX < mouse_x_max_l &&
      mouseY > mouse_y_min_l && mouseY < mouse_y_max_l && !button_l_pressed) {
      button_l_pressed = true;
      switchStopWatchL();
    }
    if (mouseX > mouse_x_min_r && mouseX < mouse_x_max_r &&
      mouseY > mouse_y_min_r && mouseY < mouse_y_max_r && !button_r_pressed) {
      button_r_pressed = true;
      switchStopWatchR();
    }
  }
}


function renderButtons() {
  push();
  textAlign(CENTER)
  let button_l_c;
  let button_r_c;

  if (stopwatch_l_running)
    button_l_c = color(button_l_c2);
  else
    button_l_c = color(button_l_c1);

  if (stopwatch_r_running)
    button_r_c = color(button_r_c2);
  else
    button_r_c = color(button_r_c1);

  push();

  if (button_l_pressed) {
    button_l_c.setAlpha(150);
    strokeWeight(line_width * 2);
  }
  fill(button_l_c);
  rect(mn, button_y, button_width, button_height, button_round);
  fill(0);
  textSize(text_size);
  strokeWeight(line_width * 0.5);
  if (!stopwatch_l_running && !stopwatch_r_running)
    text("START", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
  else if (stopwatch_l_running)
    text("STOPP", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
  pop()
  push();
  if (button_r_pressed) {
    button_r_c.setAlpha(150);
    strokeWeight(line_width * 2);
  }
  fill(button_r_c);
  rect(width * 0.5 + mn, button_y, button_width, button_height, button_round);

  fill(0);
  textSize(text_size);
  strokeWeight(line_width * 0.5);
  if (!stopwatch_l_running && !stopwatch_r_running)
    text("START", width * 0.5 + mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
  else if (stopwatch_r_running)
    text("STOPP", width * 0.5 + mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
  pop();

  pop();
}


function switchStopWatchL() {
  if (!stopwatch_l_running && !stopwatch_r_running) {
    start_time = millis();
    stopwatch_l_running = true;
    stopwatch_r_running = true;
  } else if (stopwatch_l_running)
    stopwatch_l_running = false;
}


function switchStopWatchR() {
  if (!stopwatch_l_running && !stopwatch_r_running) {
    start_time = millis();
    stopwatch_l_running = true;
    stopwatch_r_running = true;
  } else if (stopwatch_r_running)
    stopwatch_r_running = false;
}


function myInputEvent() {
  inp_dist = this.value();
}


function mousePressed() {
  checkButtons();

  return false;
}


function touchStarted() {
  checkButtons();

  return false;
}


function mouseReleased() {
  button_l_pressed = false;
  button_r_pressed = false;
}


function touchMoved() {
  if (mouseY < button_y + button_height + mn) {
    return false;
  }
}


function windowResized() {
  resizeCanvas(windowWidth, windowHeight)
  calculateLayout();
}